//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcocrsa.cpp,v 1.9 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcocrsa.cpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------
// The RomaClientResponse public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoimsg.hpp>
#include <kxcocrs.hpp>

TRACE_MODULE_VERSION($Revision: 1.9 $);


//--------------------------------------------------------------------
// Define RTTI information.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaClientResponse);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaClientResponse::RomaClientResponse() 
{
    TRACE_ENTRY(RomaClientResponse::RomaClientResponse, << TRACE(this));

    m_impl = new RomaImplMsgIncoming;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with an exsisting internal implementation
// instance.
//--------------------------------------------------------------------
RomaClientResponse::RomaClientResponse(RomaImplMsgIncoming* impl)
    : m_impl(impl)
{
    TRACE_ENTRY(RomaClientResponse::RomaClientResponse, << TRACE(this)
                                                        << TRACE(impl));
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaClientResponse::~RomaClientResponse()
{
    TRACE_ENTRY(RomaClientResponse::~RomaClientResponse, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaClientResponse::RomaClientResponse(const RomaClientResponse& resp)
    : m_impl(resp.m_impl)
{
    TRACE_ENTRY(RomaClientResponse::RomaClientResponse, << TRACE(this)
                                                        << TRACE(resp));
    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaClientResponse& 
RomaClientResponse::operator = (const RomaClientResponse& resp)
{
    TRACE_ENTRY(RomaClientResponse::operator =, << TRACE(this)
                                                << TRACE(resp));

    if (m_impl != resp.m_impl)
    {
        m_impl->releaseInstance();
        m_impl = resp.m_impl;
        m_impl->lockInstance();
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Get the message subject.
//--------------------------------------------------------------------
const char* RomaClientResponse::subject() const
{
    return m_impl->getSubjectName();
}

//--------------------------------------------------------------------
// Get the message theme.
//--------------------------------------------------------------------
const RomaTheme& RomaClientResponse::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Get the Message ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaClientResponse::msgId() const
{
    return m_impl->getMsgId();
}

//--------------------------------------------------------------------
// Get the Correlation ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaClientResponse::correlId() const
{
    return m_impl->getCorrelId();
}

//--------------------------------------------------------------------
// Get the message priority.
//--------------------------------------------------------------------
int RomaClientResponse::priority() const
{
    return m_impl->getPriority();
}

//--------------------------------------------------------------------
// Get the message type.
//--------------------------------------------------------------------
Roma::MsgType RomaClientResponse::msgType() const
{
    return m_impl->getType();
}

//--------------------------------------------------------------------
// Get the message persistence.
//--------------------------------------------------------------------
Roma::Persistence RomaClientResponse::persistence() const
{
    return m_impl->getPersistence();
}

//--------------------------------------------------------------------
// Get the message format.
//--------------------------------------------------------------------
int RomaClientResponse::format() const
{
    return m_impl->getFormat();
}

//--------------------------------------------------------------------
// Get the message feedback.
//--------------------------------------------------------------------
int RomaClientResponse::feedback() const
{
    return m_impl->getFeedback();
}

//--------------------------------------------------------------------
// Get the message expiry.
//--------------------------------------------------------------------
int RomaClientResponse::expiry() const
{
    return m_impl->getExpiry();
}

//--------------------------------------------------------------------
// Get the message codepage.
//--------------------------------------------------------------------
int RomaClientResponse::codepage() const
{
    return m_impl->getCodepage();
}

//--------------------------------------------------------------------
// Get the message encoding.
//--------------------------------------------------------------------
int RomaClientResponse::encoding() const
{
    return m_impl->getEncoding();
}

//--------------------------------------------------------------------
// Get the message status.
//--------------------------------------------------------------------
KXCULONG RomaClientResponse::status() const
{
    return m_impl->getStatus();
}

//--------------------------------------------------------------------
// Get the message backout count.
//--------------------------------------------------------------------
int RomaClientResponse::backoutCount() const
{
    return m_impl->getBackoutCount();
}

//--------------------------------------------------------------------
// Get the message format name.
//--------------------------------------------------------------------
const char* RomaClientResponse::formatName() const
{
    return m_impl->getFormatName();
}

//--------------------------------------------------------------------
// Get the message name.
//--------------------------------------------------------------------
const char* RomaClientResponse::msgName() const
{
    return m_impl->getMsgName();
}

//--------------------------------------------------------------------
// Get the message timestamp.
//--------------------------------------------------------------------
RomaTimestamp RomaClientResponse::timestamp() const
{
    return m_impl->getTimestamp();
}

//--------------------------------------------------------------------
// Get the message data buffer.
//--------------------------------------------------------------------
void* RomaClientResponse::msgBuffer() const
{
    return m_impl->getMsgBuffer();
}

//--------------------------------------------------------------------
// Get the message data length.
//--------------------------------------------------------------------
int RomaClientResponse::msgDataLength() const
{
    return m_impl->getMsgLength();
}

//--------------------------------------------------------------------
// Get the tag data buffer.
//--------------------------------------------------------------------
void* RomaClientResponse::tagBuffer() const
{
    return m_impl->getTagBuffer();
}

//--------------------------------------------------------------------
// Get the tag data length.
//--------------------------------------------------------------------
int RomaClientResponse::tagDataLength() const
{
    return m_impl->getTagLength();
}

//--------------------------------------------------------------------
// Get ILS data.
//--------------------------------------------------------------------
RomaILSData* RomaClientResponse::getILSData() const
{
    return m_impl->getILSData();
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaClientResponse::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaClientResponse, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

