//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcogtoa.cpp,v 1.6 2003/02/21 22:48:04 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcogtoa.cpp,v $
// $Revision: 1.6 $
//--------------------------------------------------------------------
// The RomaGetOptions public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoiopt.hpp>
#include <kxcogto.hpp>

TRACE_MODULE_VERSION($Revision: 1.6 $);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaGetOptions::RomaGetOptions()
{
    TRACE_ENTRY(RomaGetOptions::RomaGetOptions, << TRACE(this));

    m_impl = new RomaImplGetOptions;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaGetOptions::RomaGetOptions(const RomaGetOptions& options)
{
    TRACE_ENTRY(RomaGetOptions::RomaGetOptions, << TRACE(this)
                                                << TRACE(options));

    m_impl = new RomaImplGetOptions(*options.m_impl);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the theme, the message ID and the
// correlation ID of the specified message.
//--------------------------------------------------------------------
RomaGetOptions::RomaGetOptions(const RomaClientRequest& request)
{
    TRACE_ENTRY(RomaGetOptions::RomaGetOptions, << TRACE(this)
                                                << TRACE(request));

    m_impl = new RomaImplGetOptions;
    m_impl->setMsgId(request.msgId());
    m_impl->setCorrelId(request.correlId());
    m_impl->setTheme(request.theme());
    m_impl->setUseTheme(true);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaGetOptions::~RomaGetOptions()
{
    TRACE_ENTRY(RomaGetOptions::~RomaGetOptions, << TRACE(this));

    delete m_impl;
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaGetOptions&
RomaGetOptions::operator = (const RomaGetOptions& options)
{
    *m_impl = *options.m_impl;
    return *this;
}

//--------------------------------------------------------------------
// Specify true to allow native messages delivery.
//--------------------------------------------------------------------
void RomaGetOptions::allowNative(bool f)
{
    m_impl->setAllowNative(f);
}

//--------------------------------------------------------------------
// Set the signal.
//--------------------------------------------------------------------
void RomaGetOptions::setSignal(KXCULONG signal)
{
    m_impl->setSignal(signal);
}

//--------------------------------------------------------------------
// Unset the signal.
//--------------------------------------------------------------------
void RomaGetOptions::unsetSignal()
{
    m_impl->useSignal(false);
}

//--------------------------------------------------------------------
// Set the termination indicator.
//--------------------------------------------------------------------
void RomaGetOptions::setTermSupported(bool f)
{
    m_impl->setTermSupported(f);
}

//--------------------------------------------------------------------
// Specify true to suppress error profile use.
//--------------------------------------------------------------------
void RomaGetOptions::setSuppressErrorProfile(bool f)
{
    m_impl->setSuppressErrorProfile(f);
}

//--------------------------------------------------------------------
// Set ILS data.
//--------------------------------------------------------------------
void RomaGetOptions::setILSData(const RomaILSData& data)
{
    m_impl->setILSData(data);
}

//--------------------------------------------------------------------
// Set the security definition.
//--------------------------------------------------------------------
void RomaGetOptions::setSecurityDef(const RomaSecurityDef& def)
{
    m_impl->setSecurityDef(def);
}

//--------------------------------------------------------------------
// Set the error profile name.
//--------------------------------------------------------------------
void RomaGetOptions::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//--------------------------------------------------------------------
// Set the API profile name.
//--------------------------------------------------------------------
void RomaGetOptions::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}

//--------------------------------------------------------------------
// Set the operation.
//--------------------------------------------------------------------
void RomaGetOptions::setOperation(Roma::Operation op)
{
    m_impl->setOperation(op);
}

//--------------------------------------------------------------------
// Set the put selector.
//--------------------------------------------------------------------
void RomaGetOptions::setSelector(Roma::GetSelector sel)
{
    m_impl->setSelector(sel);
}

//--------------------------------------------------------------------
// Indicate whether the theme must be closed.
//--------------------------------------------------------------------
void RomaGetOptions::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

//--------------------------------------------------------------------
// Indicate whether the theme must be retained.
//--------------------------------------------------------------------
void RomaGetOptions::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

//--------------------------------------------------------------------
// Specify true to filter messages by the theme.
//--------------------------------------------------------------------
void RomaGetOptions::setUseTheme(bool f)
{
    m_impl->setUseTheme(f);
}

//--------------------------------------------------------------------
// Specify true to retrieve messages with any theme.
//--------------------------------------------------------------------
void RomaGetOptions::setAnyTheme(bool f)
{
    m_impl->setAnyTheme(f);
}

//--------------------------------------------------------------------
// Set the theme.
//--------------------------------------------------------------------
void RomaGetOptions::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Set the Message ID.
//--------------------------------------------------------------------
void RomaGetOptions::setMsgId(const RomaMsgId& mid)
{
    m_impl->setMsgId(mid);
}

//--------------------------------------------------------------------
// Set the Correlation ID.
//--------------------------------------------------------------------
void RomaGetOptions::setCorrelId(const RomaMsgId& cid)
{
    m_impl->setCorrelId(cid);
}

//--------------------------------------------------------------------
// Set the wait interval.
//--------------------------------------------------------------------
void RomaGetOptions::setWaitInterval(int v)
{
    m_impl->setWaitInterval(v);
}

//--------------------------------------------------------------------
// Specify true to wait for an incoming message indefinite.
//--------------------------------------------------------------------
void RomaGetOptions::setWaitIndefinite(bool f)
{
    m_impl->setWaitIndefinite(f);
}

//--------------------------------------------------------------------
// Specify true to prohibit an EMS invocation on an error.
//--------------------------------------------------------------------
void RomaGetOptions::setNoEms(bool f)
{
    m_impl->setNoEms(f);
}

//--------------------------------------------------------------------
// Specify true to prohibit notifications.
//--------------------------------------------------------------------
void RomaGetOptions::setNoNotification(bool f)
{
    m_impl->setNoNotification(f);
}

//--------------------------------------------------------------------
// Set the message subject style.
//--------------------------------------------------------------------
void RomaGetOptions::setSubjectStyle(Roma::Style style)
{
    m_impl->setSubjectStyle(style);
}

//--------------------------------------------------------------------
// Set the syncpoint definition.
//--------------------------------------------------------------------
void RomaGetOptions::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaGetOptions::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaGetOptions, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

