//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoils.cpp,v 1.9 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoils.cpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------
// The RomaILSData public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoigdt.hpp>
#include <kxcoils.hpp>

TRACE_MODULE_VERSION($Revision: 1.9 $);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaILSData::RomaILSData()
{
    TRACE_ENTRY(RomaILSData::RomaILSData, << TRACE(this));

    m_impl = new RomaImplILSData;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the specified implementation.
// For internal use only.
//--------------------------------------------------------------------
RomaILSData::RomaILSData(RomaImplILSData* impl) : m_impl(impl)
{
    TRACE_ENTRY(RomaILSData::RomaILSData, << TRACE(this) 
                                          << TRACE(impl));
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
// It creates the shallow copy of the specified object.
// The source object and its copy share the same internal
// implementation instance.
//--------------------------------------------------------------------
RomaILSData::RomaILSData(const RomaILSData& data) : m_impl(data.m_impl)
{
    TRACE_ENTRY(RomaILSData::RomaILSData(const RomaILSData&), 
        << TRACE(this) << TRACE(data.m_impl));

    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

RomaILSData& RomaILSData::operator=(const RomaILSData& data)
{
    KXCO_ASSIGN_SHARED(data);
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaILSData::~RomaILSData()
{
    TRACE_ENTRY(RomaILSData::~RomaILSData, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Indicate whether an ILS data replacement is allowed.
//--------------------------------------------------------------------
void RomaILSData::setReplace(bool replace)
{
    m_impl->setReplace(replace);
}

//--------------------------------------------------------------------
// Set data for the specified ILS method.
//--------------------------------------------------------------------
bool RomaILSData::setBuffer(void* buffer, 
                            int length, 
                            int method,
                            Roma::ILSType type)
{
    return m_impl->setBuffer(buffer, length, method, type);
}

//--------------------------------------------------------------------
// Get the first ILS data buffer.
//--------------------------------------------------------------------
const void* RomaILSData::getBuffer(int& length,
                                   int id,
                                   Roma::ILSType type)
{
    return m_impl->getBuffer(length, id, type);
}

//--------------------------------------------------------------------
// Get the next ILS data buffer.
//--------------------------------------------------------------------
const void* RomaILSData::getNextBuffer(int& length, 
                                       int& id, 
                                       Roma::ILSType& type)
{
    return m_impl->getNextBuffer(length, id, type);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaILSData::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaILSData, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

