//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoilst.hpp,v 1.4 2004/02/18 19:56:36 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoilst.hpp,v $
// $Revision: 1.4 $
//--------------------------------------------------------------------
// The single linked list template class.
//--------------------------------------------------------------------

#ifndef __KXCOILST_HPP__
#define __KXCOILST_HPP__


//--------------------------------------------------------------------
// List item template.
//--------------------------------------------------------------------
template <class Item>
class RomaImplListItem
{
public:
    RomaImplListItem() : m_next(NULL) {}

    Item* getNextItem() { return m_next; }
    void setNextItem(Item* i) { m_next = i; }

private:
    RomaImplListItem(const RomaImplListItem<Item>&);
    RomaImplListItem<Item>& operator=(const RomaImplListItem<Item>&);

    Item* m_next;
};


//--------------------------------------------------------------------
// Single linked list template.
//--------------------------------------------------------------------
template <class Item>
class RomaImplList
{
public:
    RomaImplList() : m_first(NULL)
    {
    }


    virtual ~RomaImplList() 
    { 
        clear(); 
    }


    RomaImplList(const RomaImplList<Item>& list) : m_first(NULL)
    {
        Item* src = list.m_first;
        Item* dst = NULL;

        for (; src != NULL; src = src->getNextItem())
        {
            Item* i = new Item(*src);

            if (m_first == NULL) m_first = i;
            else dst->setNextItem(i);

            dst = i;
        }
    }


    RomaImplList<Item>& operator=(const RomaImplList<Item>& list)
    {
        if (this != &list)
        {
            clear();

            Item* src = list.m_first;
            Item* dst = NULL;

            for (; src != NULL; src = src->getNextItem())
            {
                Item* i = new Item(*src);

                if (m_first == NULL) m_first = i;
                else dst->setNextItem(i);

                dst = i;
            }
        }
    
        return *this;
    }


    bool isEmpty() const 
    { 
        return (m_first == NULL); 
    }


    unsigned getCount() const
    {
        unsigned c = 0;
        for (Item* next = m_first; next != NULL; 
            next = next->getNextItem(), ++c);
        return c;
    }


    Item* getAt(unsigned pos)
    {
        Item* result = m_first;

        for (unsigned i = 0; result != NULL; ++i)
        {
            if (i == pos)
            {
                result->lockInstance();
                return result;
            }

            result = result->getNextItem();
        }

        return NULL;
    }


    bool add(Item* i)
    {
        if (i == NULL) return false;

        if (m_first == NULL)
        {
            m_first = new Item(*i);
            return true;
        }

        Item* cur = m_first;
        Item* next = m_first->getNextItem();

        while (next != NULL)
        {
            cur = next;
            next = cur->getNextItem();
        }

        cur->setNextItem(new Item(*i));
        return true;
    }


    bool insert(Item* i, unsigned pos)
    {
        if (i == NULL) return false;

        if (!pos)
        {
            Item* x = new Item(*i);
            x->setNextItem(m_first);
            m_first = x;
            return true;
        }
    
        unsigned ci = 1;
        Item* cur = m_first;

        while (ci != pos && cur != NULL)
        {
            cur = cur->getNextItem();
            ++ci;
        }

        if (cur == NULL) return false;

        Item* x = new Item(*i);
        x->setNextItem(cur->getNextItem());
        cur->setNextItem(x);
        return true;
    }


    bool remove(Item* i)
    {
        if (i == NULL || m_first == NULL) return false;

        if (i == m_first)
        {
            Item* next = m_first->getNextItem();
            m_first->setNextItem(NULL);
            m_first->releaseInstance();
            m_first = next;
            return true;
        }

        Item* prev = m_first;
        Item* cur = m_first->getNextItem();

        while (cur != NULL)
        {
            if (cur == i)
            {
                Item* next = cur->getNextItem();
                prev->setNextItem(next);
                cur->setNextItem(NULL);
                cur->releaseInstance();
                return true;
            }

            prev = cur;
            cur = cur->getNextItem();
        }

        return false;
    }


    bool remove(unsigned pos)
    {
        if (m_first == NULL) return false;

        if (!pos)
        {
            Item* next = m_first->getNextItem();
            m_first->setNextItem(NULL);
            m_first->releaseInstance();
            m_first = next;
            return true;
        }

        unsigned ci = 1;
        Item* prev = m_first;
        Item* cur = m_first->getNextItem();

        while (cur != NULL)
        {
            if (ci == pos)
            {
                Item* next = cur->getNextItem();
                prev->setNextItem(next);
                cur->setNextItem(NULL);
                cur->releaseInstance();
                return true;
            }

            prev = cur;
            cur = cur->getNextItem();
            ++ci;
        }

        return false;
    }


    void clear()
    {
        Item* cur = m_first;

        while (cur != NULL)
        {
            Item* next = cur->getNextItem();
            cur->setNextItem(NULL);
            cur->releaseInstance();
            cur = next;
        }

        m_first = NULL;
    }


    Item* getRoot()
    {
        return m_first;
    }


protected:
    Item* m_first;
};


//--------------------------------------------------------------------
// Simple list iterator template.
//--------------------------------------------------------------------
template <class Item, class List>
class RomaImplListIter
{
public:
    explicit RomaImplListIter(List& list)
        : m_list(&list), m_item(list.getRoot())
    {
        m_list->lockInstance();
        if (m_item != NULL) m_item->lockInstance();
    }

    RomaImplListIter(const RomaImplListIter<Item,List>& it)
        : m_list(it.m_list), m_item(it.m_item)
    {
        m_list->lockInstance();
        if (m_item != NULL) m_item->lockInstance();
    }

    RomaImplListIter<Item,List>& operator=(
        const RomaImplListIter<Item,List>& it)
    {
        if (this != &it)
        {
            if (m_item != NULL) m_item->releaseInstance();
            m_item = it.m_item;
            if (m_item != NULL) m_item->lockInstance();

            m_list->releaseInstance();
            m_list = it.m_list;
            m_list->lockInstance();
        }

        return *this;
    }

    virtual ~RomaImplListIter()
    {
        if (m_item != NULL) m_item->releaseInstance();
        m_list->releaseInstance();
    }

    Item* getNext()
    {
        if (m_item == NULL)
        {
            return NULL;
        }

        Item* ret = m_item;
        m_item = ret->getNextItem();
        if (m_item != NULL) m_item->lockInstance();
        return ret;
    }

private:
    List* m_list;
    Item* m_item;
};


#endif

