//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoimsg.hpp,v 1.22 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoimsg.hpp,v $
// $Revision: 1.22 $
//--------------------------------------------------------------------
// The internal implementation of various Roma messages.
//--------------------------------------------------------------------

#ifndef __KXCOIMSG_HPP__
#define __KXCOIMSG_HPP__

#include "kxcotrb.hpp"
#include "kxcomid.hpp"
#include "kxcots.hpp"
#include "kxcosub.hpp"
#include "kxcoils.hpp"
#include "kxcothm.hpp"
#include "kxcosec.hpp"
#include "kxcoigdt.hpp"
#include "kxcoiopt.hpp"


//--------------------------------------------------------------------
// The RomaImplMessage class.
// This is the base class for all message objects.
//--------------------------------------------------------------------
class KXCODLL RomaImplMessage : public RomaImplShareable, 
                                public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplMessage();

    //----------------------------------------------------------------
    // Copy constructor.
    //----------------------------------------------------------------
    RomaImplMessage(const RomaImplMessage&);

    //----------------------------------------------------------------
    // Constructor creating the message from the KXCMSG structure.
    //----------------------------------------------------------------
    RomaImplMessage(const KXCMSG&, HKXCILS, RomaImplContext*);

    //----------------------------------------------------------------
    // Operator casting the object to a pointer to the KXCMSG.
    //----------------------------------------------------------------
    operator PKXCMSG() const { return (PKXCMSG)&m_kxcmsg; }

    //----------------------------------------------------------------
    // Get/set a message buffer.
    //----------------------------------------------------------------
    bool setMsgBuffer(const void*, int);
    void* getMsgBuffer() const { return (void*)m_kxcmsg.MsgBuffer.pBuffer; }
    int getMsgLength() const { return m_kxcmsg.MsgLen; }

    //----------------------------------------------------------------
    // Get/set a tag buffer.
    //----------------------------------------------------------------
    bool setTagBuffer(const void*, int);
    void* getTagBuffer() const { return (void*)m_kxcmsg.TagBuffer.pBuffer; }
    int getTagLength() const { return m_kxcmsg.TagLen; }

    //----------------------------------------------------------------
    // Get/set a Message ID.
    //----------------------------------------------------------------
    void setMsgId(const RomaMsgId& id) { m_msgId = id; }
    const RomaMsgId& getMsgId() { return m_msgId; }

    //----------------------------------------------------------------
    // Get/set a Correlation ID.
    //----------------------------------------------------------------
    void setCorrelId(const RomaMsgId& id) { m_correlId = id; }
    const RomaMsgId& getCorrelId() { return m_correlId; }

    //----------------------------------------------------------------
    // Get the message type.
    //----------------------------------------------------------------
    Roma::MsgType getType() const { 
        return (Roma::MsgType)m_kxcmsg.MsgType; }

    //----------------------------------------------------------------
    // Get the message status.
    //----------------------------------------------------------------
    KXCULONG getStatus() const { return m_kxcmsg.Status; }

    //----------------------------------------------------------------
    // Get the message backout count.
    //----------------------------------------------------------------
    KXCULONG getBackoutCount() const { return m_kxcmsg.BackoutCount; }

    //----------------------------------------------------------------
    // Get the message timestamp.
    //----------------------------------------------------------------
    RomaTimestamp getTimestamp() const { 
        return RomaTimestamp(m_kxcmsg.TimeStamp.Seconds, 
            m_kxcmsg.TimeStamp.Millisecs); }

    //----------------------------------------------------------------
    // Get/set the message priority.
    //----------------------------------------------------------------
    void setPriority(KXCULONG val) { m_kxcmsg.Priority = val; }
    KXCULONG getPriority() const { return m_kxcmsg.Priority; }

    //----------------------------------------------------------------
    // Get/set the message persistence.
    //----------------------------------------------------------------
    void setPersistence(Roma::Persistence val) { 
        m_kxcmsg.Persistence = val; }
    Roma::Persistence getPersistence() const {
        return (Roma::Persistence)m_kxcmsg.Persistence; }

    //----------------------------------------------------------------
    // Get/set the message format.
    //----------------------------------------------------------------
    void setFormat(KXCULONG format) { m_kxcmsg.Format = format; }
    KXCULONG getFormat() const { return m_kxcmsg.Format; }

    //----------------------------------------------------------------
    // Get/set the message feedback.
    //----------------------------------------------------------------
    void setFeedback(KXCULONG val) { m_kxcmsg.Feedback = val; }
    KXCULONG getFeedback() const { return m_kxcmsg.Feedback; }

    //----------------------------------------------------------------
    // Get/set the message expiry.
    //----------------------------------------------------------------
    void setExpiry(KXCULONG val) { m_kxcmsg.Expiry = val; }
    KXCULONG getExpiry() const { return m_kxcmsg.Expiry; }

    //----------------------------------------------------------------
    // Get/set the message codepage.
    //----------------------------------------------------------------
    void setCodepage(KXCULONG cp) { m_kxcmsg.CodePage = cp; }
    KXCULONG getCodepage() const { return m_kxcmsg.CodePage; }

    //----------------------------------------------------------------
    // Get/set the message encoding.
    //----------------------------------------------------------------
    void setEncoding(KXCULONG enc) { m_kxcmsg.Encoding = enc; }
    KXCULONG getEncoding() const { return m_kxcmsg.Encoding; }

    //----------------------------------------------------------------
    // Get/set the message format name.
    //----------------------------------------------------------------
    const char* getFormatName() const { 
        return (const char*)m_kxcmsg.FormatName; }
    void setFormatName(const char* name) { 
        kxcSetMsgFormatName(&m_kxcmsg, name); }

    //----------------------------------------------------------------
    // Get/set the message name.
    //----------------------------------------------------------------
    const char* getMsgName() const { 
        return (const char*)m_kxcmsg.MessageName; }
    void setMsgName(const char* name) { 
        kxcSetMsgMsgName(&m_kxcmsg, name); }

    //----------------------------------------------------------------
    // Get the message subject name.
    //----------------------------------------------------------------
    const char* getSubjectName() const { 
        return (const char*)m_kxcmsg.SubjectName; }

    //----------------------------------------------------------------
    // Get ILS data.
    //----------------------------------------------------------------
    RomaILSData* getILSData();

    //----------------------------------------------------------------
    // Enable the application-independent data buffers management
    // mode.
    //----------------------------------------------------------------
    void beSmart() { m_smart = true; }

    //RomaImplContext* getContext() { return m_context; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Assignment operator.
    //----------------------------------------------------------------
    RomaImplMessage& operator = (const RomaImplMessage&);

    //----------------------------------------------------------------
    // Protected destructor.
    // The object should be deleted using the releaseInstance() method
    // only.
    //----------------------------------------------------------------
    virtual ~RomaImplMessage();
    
    //----------------------------------------------------------------
    // Free dynamically allocated memory of the object.
    //----------------------------------------------------------------
    virtual void clear();

    //----------------------------------------------------------------
    // Free dynamically allocated memory of the buffer.
    //----------------------------------------------------------------
    void freeBuffer(KXCBUFFER&);

    //----------------------------------------------------------------
    // Copy data to the buffer.
    //----------------------------------------------------------------
    bool setBuffer(KXCBUFFER&, const void*, int);

    //----------------------------------------------------------------
    // The message data structure.
    //----------------------------------------------------------------
    KXCMSG m_kxcmsg;

    //----------------------------------------------------------------
    // Message ID.
    //----------------------------------------------------------------
    RomaMsgId m_msgId;

    //----------------------------------------------------------------
    // Correlation ID.
    //----------------------------------------------------------------
    RomaMsgId m_correlId;

    //----------------------------------------------------------------
    // Incoming ILS data handle.
    //----------------------------------------------------------------
    HKXCILS m_ilsIncoming;

    //----------------------------------------------------------------
    // Indicates whether data buffers of the message must be created
    // and owned by the object. This feature used by the ActiveX API.
    // By default, data buffers are provided and owned by an
    // application.
    //----------------------------------------------------------------
    bool m_smart;

    //----------------------------------------------------------------
    // Indicates whether the message buffer is owned by the API.
    //----------------------------------------------------------------
    bool m_msgCopy;

    //----------------------------------------------------------------
    // Indicates whether the tag buffer is owned by the API.
    //----------------------------------------------------------------
    bool m_tagCopy;

    //----------------------------------------------------------------
    // Source context.
    //----------------------------------------------------------------
    RomaImplContext* m_context;
    HKXC m_bufOwner;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplMsgIncoming class.
// This is the base class for incoming messages.
//--------------------------------------------------------------------
class KXCODLL RomaImplMsgIncoming : public RomaImplMessage
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplMsgIncoming();

    //----------------------------------------------------------------
    // Constructor creating the message from the KXCMSG structure.
    //----------------------------------------------------------------
    RomaImplMsgIncoming(const KXCMSG&,
                        HKXCTHEME, 
                        HKXCILS, 
                        RomaImplContext&);

    //----------------------------------------------------------------
    // Get/set the message theme.
    //----------------------------------------------------------------
    const RomaTheme& getTheme() const { return m_theme; }
    void setTheme(const RomaTheme& theme) { m_theme = theme; }
    void setTheme(HKXCTHEME);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Assignment operator.
    //----------------------------------------------------------------
    RomaImplMsgIncoming& operator = (const RomaImplMsgIncoming&);

    //----------------------------------------------------------------
    // Message theme.
    //----------------------------------------------------------------
    RomaTheme m_theme;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplMsgOutgoing class.
// This is the base class for outgoing messages.
//--------------------------------------------------------------------
class KXCODLL RomaImplMsgOutgoing : public RomaImplMessage, 
                                    public RomaImplCommonPutOptions
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplMsgOutgoing();

    //----------------------------------------------------------------
    // Set the security definition.
    //----------------------------------------------------------------
    void setSecurityDef(const RomaSecurityDef&);

    //----------------------------------------------------------------
    // Fill the C API structures with object data.
    //----------------------------------------------------------------
    virtual void populate(KXCPUTOPT&, KXCSYNC&, HKXCTHEME&) const;

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

    void setIncomingILSData(HKXCILS, RomaImplContext*);

protected:
    //----------------------------------------------------------------
    // Security definition.
    //----------------------------------------------------------------
    RomaSecurityDef m_security;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplMsgOutgoing(const RomaImplMsgOutgoing&);
    RomaImplMsgOutgoing& operator = (const RomaImplMsgOutgoing&);

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplClientRequest class.
// This is the internal implementation of the RomaClientRequest public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplClientRequest : public RomaImplMsgOutgoing
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplClientRequest();

    //----------------------------------------------------------------
    // Set/get the business service name.
    //----------------------------------------------------------------
    void setServiceName(const char*);

    const char* getServiceName() const { 
        return (*m_service ? (const char*)m_service : NULL); }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Business service name.
    //----------------------------------------------------------------
    KXCOBJNAMESZ m_service;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplClientRequest(const RomaImplClientRequest&);
    RomaImplClientRequest& operator = (const RomaImplClientRequest&);

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplElementResponse class.
// This is the internal implementation of the RomaBeResponse public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplElementResponse : public RomaImplMsgOutgoing 
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplElementResponse(const RomaTheme& = RomaTheme());

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplElementResponse(const RomaImplElementResponse&);
    RomaImplElementResponse& operator = (const RomaImplElementResponse&);
};


//--------------------------------------------------------------------
// The RomaImplStorable class.
// This is the internal implementation of the RomaStorable public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplStorable : public RomaImplMsgIncoming
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplStorable();

    //----------------------------------------------------------------
    // Constructor creating the message from the KXCMSG structure.
    //----------------------------------------------------------------
    RomaImplStorable(const KXCMSG&, HKXCTHEME, RomaImplContext&);

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaImplStorable(const RomaImplStorable&);
    RomaImplStorable& operator = (const RomaImplStorable&);

    //----------------------------------------------------------------
    // Fill the C API structures with object data.
    //----------------------------------------------------------------
    void populate(KXCPUTOPT&, HKXCTHEME&) const;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplExceptionMessage class.
// This is the internal implementation of the RomaExceptionMessage
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplExceptionMessage : public RomaImplMessage
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplExceptionMessage();

    //----------------------------------------------------------------
    // Constructor creating the message from the KXCMSG structure.
    //----------------------------------------------------------------
    RomaImplExceptionMessage(const RomaImplMessage&);

    //----------------------------------------------------------------
    // Get/set the message theme.
    //----------------------------------------------------------------
    const RomaTheme& getTheme() const { return m_theme; }
    void setTheme(const RomaTheme& theme) { m_theme = theme; }

    //----------------------------------------------------------------
    // Get/set exception elements.
    //----------------------------------------------------------------
    void setElements(KXCULONG elems) { m_elements = elems; }
    KXCULONG getElements() const { return m_elements; }

    //----------------------------------------------------------------
    // Set error information.
    //----------------------------------------------------------------
    void setCallType(Roma::CallType);
    void setStatus(Roma::ErrorStatus);
    void setCompCode(Roma::ErrorSubclass, Roma::CompCode);
    void setReturnCode(Roma::ErrorSubclass, int);
    void setReason(Roma::ErrorSubclass, int);
    void setOperationType(Roma::ErrorSubclass, int);
    void setPosition(Roma::ErrorSubclass, int);
    void setErrorType(Roma::ErrorSubclass, Roma::ErrorType);
    void setAdditionalInfo(Roma::ErrorSubclass, const char*);
    bool setUserBuffer(const void*, int);

    //----------------------------------------------------------------
    // Fill the C API structures with object data.
    //----------------------------------------------------------------
    void populate(KXCEXCEPTION&) const;
    void populate(KXCPUTOPT&) const;

    //----------------------------------------------------------------
    // Save the theme on the message.
    //----------------------------------------------------------------
    void capture(HKXCTHEME);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Protected destructor.
    // The object should be deleted using the releaseInstance() method
    // only.
    //----------------------------------------------------------------
    virtual ~RomaImplExceptionMessage();

    //----------------------------------------------------------------
    // Free dynamically allocated memory of the object.
    //----------------------------------------------------------------
    virtual void clear();

    //----------------------------------------------------------------
    // Exception data.
    //----------------------------------------------------------------
    KXCEXCEPTION m_data;

    //----------------------------------------------------------------
    // Exception elements.
    //----------------------------------------------------------------
    KXCULONG m_elements;

    //----------------------------------------------------------------
    // Message theme.
    //----------------------------------------------------------------
    RomaTheme m_theme;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplExceptionMessage(const RomaImplExceptionMessage&);
    RomaImplExceptionMessage& operator = (
        const RomaImplExceptionMessage&);

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplPublication class.
// This is the internal implementation of the RomaPublication
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplPublication : public RomaImplClientRequest
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplPublication();

    //----------------------------------------------------------------
    // Set/get the subject.
    //----------------------------------------------------------------
    void setSubject(const RomaSubject& s) { m_subject = s; }
    const RomaSubject& getSubject() const { return m_subject; }

    //----------------------------------------------------------------
    // Indicate whether the publication must be retained.
    //----------------------------------------------------------------
    void setRetain(bool b) { m_retain = b; }
    bool getRetain() const { return m_retain; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Publication subject.
    //----------------------------------------------------------------
    RomaSubject m_subject;

    //----------------------------------------------------------------
    // Indicates whether the publication must be retained.
    //----------------------------------------------------------------
    bool m_retain;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplPublication(const RomaImplPublication&);
    RomaImplPublication& operator = (const RomaImplPublication&);

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplSubscription class.
// This is the internal implementation of the RomaSubscription
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplSubscription : public RomaImplClientRequest
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplSubscription();

    //----------------------------------------------------------------
    // Set/get the subject.
    //----------------------------------------------------------------
    void setSubject(const RomaSubject& s) { m_subject = s; }
    const RomaSubject& getSubject() const { return m_subject; }

    //----------------------------------------------------------------
    // Indicate whether new publications only must be delivered.
    //----------------------------------------------------------------
    void setNewPubsOnly(bool b) { m_newPubsOnly = b; }
    bool getNewPubsOnly() const { return m_newPubsOnly; }

    //----------------------------------------------------------------
    // Indicate whether publications must be delivered by a request
    // only.
    //----------------------------------------------------------------
    void setOnRequest(bool b) { m_onRequest = b; }
    bool getOnRequest() const { return m_onRequest; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Subscription subject.
    //----------------------------------------------------------------
    RomaSubject m_subject;

    //----------------------------------------------------------------
    // Indicates whether new publications only must be delivered.
    //----------------------------------------------------------------
    bool m_newPubsOnly;

    //----------------------------------------------------------------
    // Indicates whether publications must be delivered by a request
    // only.
    //----------------------------------------------------------------
    bool m_onRequest;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplSubscription(const RomaImplSubscription&);
    RomaImplSubscription& operator = (const RomaImplSubscription&);

    friend class KxcoStreamer;
};


#endif // __KXCOIMSG_HPP__

