//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcorvoa.cpp,v 1.11 2003/02/21 22:48:07 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcorvoa.cpp,v $
// $Revision: 1.11 $
//--------------------------------------------------------------------
// The RomaRetrieveOptions public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoiopt.hpp>
#include <kxcorvo.hpp>

TRACE_MODULE_VERSION($Revision: 1.11 $);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaRetrieveOptions::RomaRetrieveOptions()
{
    TRACE_ENTRY(RomaRetrieveOptions ctor, << TRACE(this));

    m_impl = new RomaImplRetrieveOptions;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaRetrieveOptions::~RomaRetrieveOptions()
{
    TRACE_ENTRY(RomaRetrieveOptions dtor, << TRACE(this));

    delete m_impl;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaRetrieveOptions::RomaRetrieveOptions(const RomaRetrieveOptions& opt)
{
    m_impl = new RomaImplRetrieveOptions(*opt.m_impl);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaRetrieveOptions&
RomaRetrieveOptions::operator = (const RomaRetrieveOptions& opt)
{
    *m_impl = *opt.m_impl;
    return *this;
}

//--------------------------------------------------------------------
// Set an error profile name.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//--------------------------------------------------------------------
// Set an API profile name.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}

//--------------------------------------------------------------------
// Set a type of the operation.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setOperation(Roma::Operation op)
{
    m_impl->setOperation(op);
}

//--------------------------------------------------------------------
// Set a message selector.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setSelector(Roma::GetSelector sel)
{
    m_impl->setSelector(sel);
}

//--------------------------------------------------------------------
// Set a message ID.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setMsgId(const RomaMsgId& mid)
{
    m_impl->setMsgId(mid);
}

//--------------------------------------------------------------------
// Set a correlation ID.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setCorrelId(const RomaMsgId& cid)
{
    m_impl->setCorrelId(cid);
}

//--------------------------------------------------------------------
// Set a message theme.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Specify true if the message theme must be closed on retrieval.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

//--------------------------------------------------------------------
// Specify true if the message theme must not be closed on retrieval.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

//--------------------------------------------------------------------
// Specify true if only messages with the specified theme must be
// retrieved.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setUseTheme(bool f)
{
    m_impl->setUseTheme(f);
}

//--------------------------------------------------------------------
// Specify true if messages with any theme must be retrieved.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setAnyTheme(bool f)
{
    m_impl->setAnyTheme(f);
}

//--------------------------------------------------------------------
// Set a number of milliseconds of the timeout.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setWaitInterval(int v)
{
    m_impl->setWaitInterval(v);
}

//--------------------------------------------------------------------
// Specify true if an application should wait for a message
// indefinitely.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setWaitIndefinite(bool f)
{
    m_impl->setWaitIndefinite(f);
}

//--------------------------------------------------------------------
// Specify true to disable EMS invocation on an error.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setNoEms(bool f)
{
    m_impl->setNoEms(f);
}

//--------------------------------------------------------------------
// Specify true to disable notification.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setNoNotification(bool f)
{
    m_impl->setNoNotification(f);
}

//--------------------------------------------------------------------
// Set a syncpoint definition.
//--------------------------------------------------------------------
void RomaRetrieveOptions::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaRetrieveOptions::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaRetrieveOptions, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

