//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosbla.cpp,v 1.9 2003/02/21 22:48:07 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosbla.cpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------
// The RomaStorable public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoimsg.hpp>
#include <kxcosbl.hpp>
#include <kxcots.hpp>

TRACE_MODULE_VERSION($Revision: 1.9 $);

//--------------------------------------------------------------------
// Define RTTI information.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaStorable);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaStorable::RomaStorable()
{
    TRACE_ENTRY(RomaStorable::RomaStorable, << TRACE(this));

    m_impl = new RomaImplStorable;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor initializing the message with the specified
// message data.
//--------------------------------------------------------------------
RomaStorable::RomaStorable(const void* data, int length)
{
    TRACE_ENTRY(RomaStorable::RomaStorable, << TRACE(this)
                                            << TRACE(data) 
                                            << TRACE(length));
    m_impl = new RomaImplStorable;

    if (data && length)
    {
        m_impl->setMsgBuffer(data, length);
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaStorable::~RomaStorable()
{
    TRACE_ENTRY(RomaStorable::~RomaStorable, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Allows to create the object using an existing implementation
// instance.
//--------------------------------------------------------------------
RomaStorable::RomaStorable(RomaImplStorable* impl) : m_impl(impl)
{
    TRACE_ENTRY(RomaStorable::~RomaStorable, << TRACE(this)
                                             << TRACE(impl));
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaStorable::RomaStorable(const RomaStorable& msg)
{
    TRACE_ENTRY(RomaStorable::RomaStorable, << TRACE(this)
                                            << TRACE(msg));

    m_impl = new RomaImplStorable(*msg.m_impl);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaStorable& RomaStorable::operator = (const RomaStorable& msg)
{
    TRACE_ENTRY(RomaStorable::operator =, << TRACE(this)
                                          << TRACE(msg));
    *m_impl = *msg.m_impl;

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// This method can be overriden in an application to perform a message
// marshaling. The default implementation does nothing.
//--------------------------------------------------------------------
bool RomaStorable::marshalForStorage()
{
    TRACE_ENTRY(RomaStorable::marshalForStorage, << TRACE(this));
    TRACE_RETURN(true);
}

//--------------------------------------------------------------------
// Get a Roma Theme ID.
//--------------------------------------------------------------------
const RomaTheme& RomaStorable::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Set a Roma Theme ID.
//--------------------------------------------------------------------
void RomaStorable::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Get a Roma Message ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaStorable::msgId() const
{
    return m_impl->getMsgId();
}

//--------------------------------------------------------------------
// Set a Roma Message ID.
//--------------------------------------------------------------------
void RomaStorable::setMsgId(const RomaMsgId& msgId)
{
    m_impl->setMsgId(msgId);
}

//--------------------------------------------------------------------
// Get a Roma Correlation ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaStorable::correlId() const
{
    return m_impl->getCorrelId();
}

//--------------------------------------------------------------------
// Set a Roma Correlation ID.
//--------------------------------------------------------------------
void RomaStorable::setCorrelId(const RomaMsgId& correlId)
{
    m_impl->setCorrelId(correlId);
}

//--------------------------------------------------------------------
// Set the message priority.
//  A numeric value or one of the following constants:
//  PRIORITY_PROFILE - use priority specified in API profile, or
//  PRIORITY_DEFAULT - default transport priority
//--------------------------------------------------------------------
void RomaStorable::setPriority(int priority)
{
    m_impl->setPriority(priority);
}

//--------------------------------------------------------------------
// Set the message persistence to one of ...
//  PERSISTENCE_YES     - persistent
//  PERSISTENCE_NO      - not persistent
//  PERSISTENCE_PROFILE - use value in the API profile, default NO
//--------------------------------------------------------------------
void RomaStorable::setPersistence(Roma::Persistence persistence)
{
    m_impl->setPersistence(persistence);
}

//--------------------------------------------------------------------
// Set the format number.
//  If FORMAT_PROFILE is specified then format is retrieved from
//  the API profile, defaulting to FORMAT_BINARY.
//--------------------------------------------------------------------
void RomaStorable::setFormat(int format)
{
    m_impl->setFormat(format);
}

//--------------------------------------------------------------------
// Set the expiry interval.
//  If EXPIRY_PROFILE is specified then interval is retrieved from
//  the API profile, defaulting to EXPIRY_NEVER.
//--------------------------------------------------------------------
void RomaStorable::setExpiry(int expiry)
{
    m_impl->setExpiry(expiry);
}

//--------------------------------------------------------------------
// Set the codepage.
//  If CODEPAGE_PROFILE is specified then codepage number is
//  retrieved from the API profile, defaulting to CODEPAGE_USER
//--------------------------------------------------------------------
void RomaStorable::setCodepage(int codepage)
{
    m_impl->setCodepage(codepage);
}

//--------------------------------------------------------------------
// Set the encoding.
//  If ENCODING_PROFILE is specified then encoding is
//  retrieved from the API profile, defaulting to ENCODING_DEFAULT
//--------------------------------------------------------------------
void RomaStorable::setEncoding(int encoding)
{
    m_impl->setEncoding(encoding);
}

//--------------------------------------------------------------------
// Set the format name to the specified string and format name to
// FORMAT_USENAME.
//--------------------------------------------------------------------
void RomaStorable::setFormatName(const char* name)
{
    m_impl->setFormatName(name);
}

//--------------------------------------------------------------------
// Set message data.
//--------------------------------------------------------------------
bool RomaStorable::setMsgBuffer(const void* buffer, int length)
{
    return m_impl->setMsgBuffer(buffer, length);
}

//--------------------------------------------------------------------
// Get message data.
//--------------------------------------------------------------------
void* RomaStorable::msgBuffer() const
{
    return m_impl->getMsgBuffer();
}

//--------------------------------------------------------------------
// Get the length of message data.
//--------------------------------------------------------------------
int RomaStorable::msgDataLength() const
{
    return m_impl->getMsgLength();
}

//--------------------------------------------------------------------
// Get the message priority.
//  A numeric value or one of the following constants:
//  PRIORITY_PROFILE - use priority specified in API profile, or
//  PRIORITY_DEFAULT - default transport priority
//--------------------------------------------------------------------
int RomaStorable::priority() const
{
    return m_impl->getPriority();
}

//--------------------------------------------------------------------
// Get the message persistence. Values can be ...
//  PERSISTENCE_YES     - persistent
//  PERSISTENCE_NO      - not persistent
//  PERSISTENCE_PROFILE - use value in the API profile, default NO
//--------------------------------------------------------------------
Roma::Persistence RomaStorable::persistence() const
{
    return m_impl->getPersistence();
}

//--------------------------------------------------------------------
// Get the format number.
//--------------------------------------------------------------------
int RomaStorable::format() const
{
    return m_impl->getFormat();
}

//--------------------------------------------------------------------
// Get the expiry interval.
//--------------------------------------------------------------------
int RomaStorable::expiry() const
{
    return m_impl->getExpiry();
}

//--------------------------------------------------------------------
// Get the codepage.
//--------------------------------------------------------------------
int RomaStorable::codepage() const
{
    return m_impl->getCodepage();
}

//--------------------------------------------------------------------
// Get the encoding.
//--------------------------------------------------------------------
int RomaStorable::encoding() const
{
    return m_impl->getEncoding();
}

//--------------------------------------------------------------------
// Get the backout count.
//--------------------------------------------------------------------
int RomaStorable::backoutCount() const
{
    return m_impl->getBackoutCount();
}

//--------------------------------------------------------------------
// Get the format name.
//--------------------------------------------------------------------
const char* RomaStorable::formatName() const
{
    return m_impl->getFormatName();
}

//--------------------------------------------------------------------
// Get the message timestamp.
//--------------------------------------------------------------------
RomaTimestamp RomaStorable::timestamp() const
{
    return m_impl->getTimestamp();
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaStorable::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaStorable, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

