//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcotrca.hpp
//
// Declares     : internal tracing macros
//                RomaTraceStream
//                RomaTraceRadix
//
// Description  : KXC C++ Internal Tracing
//
//--------------------------------------------------------------------
// $Id: kxcotrc.hpp,v 1.10 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcotrc.hpp,v $
// $Revision: 1.10 $
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Protect against multiple includes.
//--------------------------------------------------------------------
#ifndef __KXCOTRCA_HPP
#define __KXCOTRCA_HPP

//--------------------------------------------------------------------
// Class RomaTraceable.
//--------------------------------------------------------------------
#include <kxcotrb.hpp>

//--------------------------------------------------------------------
// The RomaTraceStream class.
//
// An output text stream.
//
// This class is the simple substitution for the ostrstream, since
// ostrstream is not supported well on all platforms.
//---------------------------------------------------------------------
class KXCODLL RomaTraceStream
{
public:
    enum { MAX_LEN = 350 };

    RomaTraceStream();

    RomaTraceStream& reset();
    void write( char v );
    void write( int v );
#if defined (__alpha)
    void write( unsigned long);
#else
    void write( unsigned int );
#endif
    void write( long v );
    void write( KXCULONG v );
    void write( const char* v );
    void write( const void* v );
    const char* str();

    int m_radix;

private:
    // prohibited
    RomaTraceStream( const RomaTraceStream& );
    RomaTraceStream& operator = ( const RomaTraceStream& );

    unsigned m_pos;
    // one extra byte for '+' overflow sign and one for '\0'
    char m_buff[MAX_LEN+2];
};

//---------------------------------------------------------------------
// Output operators.
//---------------------------------------------------------------------

inline
RomaTraceStream& operator << ( RomaTraceStream& s, char v );

inline
RomaTraceStream& operator << ( RomaTraceStream& s, int v );

#if defined(__alpha)
inline
RomaTraceStream& operator << ( RomaTraceStream& s, unsigned long v);
#else
inline
RomaTraceStream& operator << ( RomaTraceStream& s, unsigned int v );
#endif

inline
RomaTraceStream& operator << ( RomaTraceStream& s, long v );

inline
RomaTraceStream& operator << ( RomaTraceStream& s, KXCULONG v );

inline
RomaTraceStream& operator << ( RomaTraceStream& s, const char* v );

inline
RomaTraceStream& operator << ( RomaTraceStream& s, const KXCCHAR* v );

inline
RomaTraceStream& operator << ( RomaTraceStream& s, const void* v );

inline RomaTraceStream&
operator << ( RomaTraceStream& s, const RomaTraceable& o );

//---------------------------------------------------------------------
// Manipulators
//---------------------------------------------------------------------
class KXCODLL RomaTraceRadix
{
public:
  RomaTraceRadix( int radix );
public:
  int m_radix;
  static const RomaTraceRadix hex;
  static const RomaTraceRadix dec;
};

inline RomaTraceStream&
operator << ( RomaTraceStream& s, const RomaTraceRadix& );

#define HEX RomaTraceRadix::hex

#define DEC RomaTraceRadix::dec

#define TRACE_HEX( x ) #x << "=" << HEX << (x) << DEC << "; "

#define TRACE( x ) #x << "=" << (x) << "; "


//------------------------------------------------------------------
// Object Model Tracing Macros
//------------------------------------------------------------------

// RAS tracing
#if defined(KXC_I_SUPPORT_KBB) || defined(KXC_I_SUPPORT_RAS)

#define WARNING_CLASS \
    (RAS1_ClassOf(Subsystem)|RAS1_ClassOf(Flow)|RAS1_ClassOf(Detail))
  
#define TRACE_MODULE_VERSION(version) \
    RAS1_ComponentUnit(RAS1CLASS_Subsystem,version,KXCOBJMODEL)

#define TRACE_ENTRY( func, args ) \
    RAS1_Entry((RAS1CLASS_Subsystem|RAS1CLASS_Public|RAS1CLASS_Flow),func);\
    RomaTraceStream ROMA_CPP_TRACE; \
    if (RAS1_IsTracing(WARNING_CLASS)) { \
    ROMA_CPP_TRACE args; \
    RAS1_Printf(RAS1, (char*)"Args - %s", ROMA_CPP_TRACE.str()); }

#define TRACE_VOID_RETURN \
    RAS1_ExitProcedure()

#define TRACE_DUMP( ptr, len, args ) \
    if (RAS1_IsTracing(WARNING_CLASS)) { \
    ROMA_CPP_TRACE.reset() args; \
    RAS1_Dump(RAS1, ptr, len, (char*)ROMA_CPP_TRACE.str()); }

#define TRACE_EVENT(x) \
    if (RAS1_IsTracing(WARNING_CLASS)) { \
    RAS1_Printf(RAS1, (char*)"!!! - %s\n", (ROMA_CPP_TRACE.reset() x).str()); }

#define TRACE_INT_RETURN(x) \
    RAS1_ExitFunction(x)

#define TRACE_RETURN(x) \
    if (RAS1_IsTracing(WARNING_CLASS)) { \
    RAS1_Printf(RAS1, (char*)"Exit: %s\n", (ROMA_CPP_TRACE.reset() << x).str()); } \
    return x;

#define TRACE_WARNING(x) \
    if (RAS1_IsTracing(WARNING_CLASS)) { \
    RAS1_Printf(RAS1, (char*)"### Warning - %s\n", \
    (ROMA_CPP_TRACE.reset() x).str()); }

#define TRACE_CLASS( x, args ) \
    #x << "<" << this << ">" << "(" args << ")"


// Use printf for tracing
#elif defined(KXC_TRACING_PRINTF)

#include <iostream.h>

#define TRACE_VERSION_COMPONENT( version, component )

#define TRACE_MODULE_VERSION(version)

#define TRACE_ENTRY( func, args )                          \
    RomaTraceStream ROMA_CPP_TRACE;                        \
    printf("%s (%s)\n", #func, (ROMA_CPP_TRACE args).str());

#define TRACE_VOID_RETURN                                  \
    printf("Exiting function.\n");                         \
    return;

#define TRACE_DUMP( ptr, len, args )                       \
    printf( "%s\n",(ROMA_CPP_TRACE.reset() args).str() );

#define TRACE_EVENT( args )                                \
    { printf( "%s\n",(ROMA_CPP_TRACE.reset() args).str() ); }

#define TRACE_INT_RETURN( x )                              \
    printf("Exiting function. Int return value: %d\n", x); \
    return (x);

#define TRACE_RETURN(x) \
    printf("Exit: %s\n", (ROMA_CPP_TRACE.reset() << (x)).str()); \
    return (x);

#define WARNING_CLASS

#define TRACE_WARNING( x )                                 \
    printf("### Warning - %s\n",(ROMA_CPP_TRACE.reset() x).str() )

#define TRACE_CLASS( x, args )                             \
    #x << "<" << this << ">" << "(" args << ")"


// Disabled tracing
#else

#define TRACE_MODULE_VERSION(version)
#define TRACE_ENTRY( func, args )
#define TRACE_VOID_RETURN
#define TRACE_DUMP( ptr, len, args )
#define TRACE_EVENT( args )

#define TRACE_INT_RETURN( x ) \
   return (x);

#define TRACE_RETURN( x ) \
    return (x);

#define WARNING_CLASS
#define TRACE_WARNING( x )

#define TRACE_CLASS( x, args ) \
    #x << "<" << this << ">" << "(" args << ")"


#endif

//--------------------------------------------------------------------
// RomaTraceStream inline methods
//--------------------------------------------------------------------

inline RomaTraceStream& RomaTraceStream::reset()
{
    m_pos = 0;
    return *this;
}

inline const char* RomaTraceStream::str()
{
    m_buff[m_pos] = 0;
    return m_buff;
}

inline RomaTraceRadix::RomaTraceRadix( int radix )
    : m_radix( radix )
{
}

inline RomaTraceStream&
operator << ( RomaTraceStream& s, const RomaTraceRadix& r )
{
    s.m_radix = r.m_radix;
    return s;
}

//--------------------------------------------------------------------
// Inlined output operators
//--------------------------------------------------------------------

inline RomaTraceStream& operator << ( RomaTraceStream& s, char ch )
{
    s.write( ch );
    return s;
}

inline RomaTraceStream& operator << ( RomaTraceStream& s, long v )
{
    s.write( v );
    return s;
}

inline
RomaTraceStream& operator << ( RomaTraceStream& s, KXCULONG v )
{
    s.write( v );
    return s;
}

inline RomaTraceStream& operator << ( RomaTraceStream& s, int v )
{
    s.write( v );
    return s;
}

#if defined(__alpha)
inline
RomaTraceStream& operator << ( RomaTraceStream& s, unsigned long v )
{
    s.write( v );
    return s;
}
#else
inline
RomaTraceStream& operator << ( RomaTraceStream& s, unsigned int v )
{
    s.write( v );
    return s;
}
#endif

inline RomaTraceStream&
operator << ( RomaTraceStream& s, const char* v )
{
    s.write( v );
    return s;
}

inline RomaTraceStream&
operator << ( RomaTraceStream& s, const void* v )
{
    s.write( v );
    return s;
}

inline
RomaTraceStream& operator << ( RomaTraceStream& s, const KXCCHAR* v )
{
    s.write( (const char*)v );
    return s;
}

inline RomaTraceStream& operator << (RomaTraceStream& s, 
                                     const RomaTraceable& o)
{
    o.trace(s);
    return s;
}


#endif

