//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcotsa.cpp,v 1.7 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcotsa.cpp,v $
// $Revision: 1.7 $
//--------------------------------------------------------------------
// The RomaTimestamp public interface.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoigdt.hpp"
#include "kxcots.hpp"

//--------------------------------------------------------------------
// Default constructor.
// Optionally accepts seconds and milliseconds to initialize
// the object.
//--------------------------------------------------------------------
RomaTimestamp::RomaTimestamp(KXCULONG seconds, KXCULONG mSeconds)
{
    m_impl = new RomaImplTimestamp(seconds, mSeconds);
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaTimestamp::RomaTimestamp(const RomaTimestamp& ts)
{
    m_impl = new RomaImplTimestamp(*ts.m_impl);
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaTimestamp::~RomaTimestamp()
{
    delete m_impl;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaTimestamp& RomaTimestamp::operator = (const RomaTimestamp& ts)
{
    *m_impl = *ts.m_impl;
    return *this;
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaTimestamp::operator == (const RomaTimestamp& ts) const
{
    return (*m_impl == *ts.m_impl);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaTimestamp::operator != (const RomaTimestamp& ts) const
{
    return (*m_impl != *ts.m_impl);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaTimestamp::operator < (const RomaTimestamp& ts) const
{
    return (*m_impl < *ts.m_impl);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaTimestamp::operator <= (const RomaTimestamp& ts) const
{
    return (*m_impl <= *ts.m_impl);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaTimestamp::operator > (const RomaTimestamp& ts) const
{
    return (*m_impl > *ts.m_impl);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaTimestamp::operator >= (const RomaTimestamp& ts) const
{
    return (*m_impl >= *ts.m_impl);
}

//--------------------------------------------------------------------
// Get seconds.
//--------------------------------------------------------------------
KXCULONG RomaTimestamp::seconds() const
{
    return m_impl->getSeconds();
}

//--------------------------------------------------------------------
// Get milliseconds.
//--------------------------------------------------------------------
KXCULONG RomaTimestamp::milliseconds() const
{
    return m_impl->getMilliseconds();
}

//--------------------------------------------------------------------
// Set seconds.
//--------------------------------------------------------------------
void RomaTimestamp::setSeconds(KXCULONG v)
{
    m_impl->setSeconds(v);
}

//--------------------------------------------------------------------
// Set milliseconds.
//--------------------------------------------------------------------
void RomaTimestamp::setMilliseconds(KXCULONG v)
{
    m_impl->setMilliseconds(v);
}

//--------------------------------------------------------------------
// Assign a value of the KXCTIMESTAMP to the object.
//--------------------------------------------------------------------
void RomaTimestamp::set(const KXCTIMESTAMP& ts)
{
    *m_impl = ts;
}

//--------------------------------------------------------------------
// Assign the object value to the KXCTIMESTAMP.
//--------------------------------------------------------------------
void RomaTimestamp::get(KXCTIMESTAMP& ts) const
{
    m_impl->populate(ts);
}

