//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcClient.java
//
// Defines      : class KxcClient
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Introduces Client abstraction in the raw interface.
* Extends the base KxcContext interface with put/get client
* operations
*/

public class KxcClient extends KxcContext
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // Initilizes new client context.
  // Configures kxcInitClient operation
  // which should be call by the user subsequently
  KxcClient( KxcOper  oper, 
             String   name, 
             int      opt, 
             byte     sec[], 
             String   modq,
             long     ilsDataHandle,
             String   id,
             String   pwd )
  {
    super( oper );
    m_op[NAME]        = name;
    m_ip[OPT]         = opt;
    m_op[SECBUF]      = sec;
    m_op[MODEL_QUEUE] = modq;
    m_ip[OP_CODE]     = NIOP_CLNINIT;
    m_lp[ILS_DATA_HANDLE] = ilsDataHandle;
    m_op[DIR_IDENTITY] = id;
    m_op[DIR_PASSWORD] = pwd;
  }
  
  // configures and returns kxcPutRequest operation.
  public KxcOper put( String bSvc )
  {
    m_op[NAME]    = bSvc;
    m_ip[OP_CODE] = NIOP_PUTREQ;
    return m_oper;
  }
  
  // configures and returns publish operation.
  public KxcOper publish( String  bSvc, 
                          int     style, 
                          String  subject,
			  int     options )
  {
    m_op[NAME]    = bSvc;
    m_op[SUBJECT] = subject;
    m_ip[STYLE]   = style;
    m_ip[OP_CODE] = NIOP_PUBLISH;
    m_ip[PUBSUBOPTS] = options;
    return m_oper;
  }
  
  // configures and returns subscribe operation.
  public KxcOper  subscribe( String bSvc, 
                             int    style, 
                             String subject,
			     int    options )
  {
    m_op[NAME]    = bSvc;
    m_op[SUBJECT] = subject;
    m_ip[STYLE]   = style;
    m_ip[OP_CODE] = NIOP_SUBSCRIBE;
    m_ip[PUBSUBOPTS] = options;
    return m_oper;
  }
  
  // configures and returns unsubscribe operation.
  public KxcOper  unsubscribe( String bSvc, 
                               int    style, 
                               String subject )
  {
    m_op[NAME]    = bSvc;
    m_op[SUBJECT] = subject;
    m_ip[STYLE]   = style;
    m_ip[OP_CODE] = NIOP_UNSUBSCRIBE;
    return m_oper;
  }

  // configures and returns delete publication operation.
  public KxcOper delPub( String bSvc , int style, String subject )
  {
    m_op[NAME]       = bSvc;
    m_op[SUBJECT]    = subject;
    m_ip[STYLE]      = style;
    m_ip[OP_CODE]    = NIOP_DELETE_PUB;
    return m_oper;
  }

  // configures and returns request update operation.
  public KxcOper reqUpd( String bSvc , int style, String subject )
  {
    m_op[NAME]       = bSvc;
    m_op[SUBJECT]    = subject;
    m_ip[STYLE]      = style;
    m_ip[OP_CODE]    = NIOP_REQ_UPDATE;
    return m_oper;
  }

  // configures and returns request unsubscribe all operation.
  public KxcOper unsubAll( String bSvc )
  {
    m_op[NAME]       = bSvc;
    m_ip[OP_CODE]    = NIOP_UNSUB_ALL;
    return m_oper;
  }

  
  // configures and returns kxcGetResponse operation.
  public KxcOper get()
  {
    m_ip[OP_CODE] = NIOP_GETRESP;
    return m_oper;
  } 
}

//--------------------------------------------------------------------
// End of file KxcClient.java.
//--------------------------------------------------------------------
