//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcobrs.hpp,v 1.13 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcobrs.hpp,v $
// $Revision: 1.13 $
//--------------------------------------------------------------------
// The RomaBeResponse public interface.
//--------------------------------------------------------------------

#ifndef __KXCOBRS_HPP
#define __KXCOBRS_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcomid.hpp>
#include <kxcothm.hpp>
#include <kxcosec.hpp>
#include <kxcosyn.hpp>
#include <kxcobrq.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplElementResponse;


//--------------------------------------------------------------------
// The RomaBeResponse class.
//
// This class represents responses made by a RomaBusinessElement
// in response to a RomaClient request. It allows an application to
// set the options required when responding to a RomaClient request.
//
// This class also provides a framework for associating application
// specific data with a BusinessElement response. The RomaBeResponse
// class can be subclassed to provide an application specific
// implementation of the marshalResponse() method.
//--------------------------------------------------------------------
class KXCODLL RomaBeResponse : public RomaTypeBase,
                               public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Construct, supplying data buffers and theme.
    //
    // Parameters:
    // [
    //   (in) theme - identifying the response
    //   (in) pointer to supplied buffer (MsgBuffer)
    //   (in) length of data within the buffer
    //   (in) pointer to supplied buffer (TagBuffer)
    //   (in) length of data within the buffer
    // ]
    //----------------------------------------------------------------
    RomaBeResponse(const RomaTheme& theme = RomaTheme());

    RomaBeResponse(const RomaTheme& theme,
                   const void* msgBuffer,
                   int         msgLength);

    RomaBeResponse(const RomaTheme& theme,
                   const void* msgBuffer,
                   int         msgLength,
                   const void* tagBuffer,
                   int         tagLength);

    //----------------------------------------------------------------
    // Construct from an existing RomaBeRequest object.
    // Initializes the response with the theme, the message ID and
    // the correlation ID of the specified request.
    //----------------------------------------------------------------
    RomaBeResponse(const RomaBeRequest* req);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaBeResponse();

    //----------------------------------------------------------------
    // Copy and assignment.
    //
    // Note: these methods don't create an independent copy of the
    // message, but just share the common implementation instance.
    //----------------------------------------------------------------
    RomaBeResponse(const RomaBeResponse&);
    RomaBeResponse& operator = (const RomaBeResponse&);

    //----------------------------------------------------------------
    // Whether theme is to be retained. (API profile may override)
    //----------------------------------------------------------------
    void setRetainTheme(bool flag = true);
    void setCloseTheme(bool flag = true);

    //----------------------------------------------------------------
    // Set the security definition to be used with this response.
    //----------------------------------------------------------------
    void setSecurityDef(const RomaSecurityDef& securityDef);

    //----------------------------------------------------------------
    // Set/get the Roma Theme ID.
    //----------------------------------------------------------------
    const RomaTheme& theme() const;
    void  setTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Set/get the Roma Message ID.
    //----------------------------------------------------------------
    const RomaMsgId& msgId() const;
    void  setMsgId(const RomaMsgId& msgId);

    //----------------------------------------------------------------
    // Set/get the Roma Correlation ID.
    //----------------------------------------------------------------
    const RomaMsgId& correlId() const;
    void  setCorrelId(const RomaMsgId& correlId);

    //----------------------------------------------------------------
    // Set the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority.
    //----------------------------------------------------------------
    void setPriority(int priority);

    //----------------------------------------------------------------
    // Set the message persistence to one of ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //----------------------------------------------------------------
    void setPersistence(Roma::Persistence persistence);

    //----------------------------------------------------------------
    // Set the format number.
    //  If FORMAT_PROFILE is specified then format is retrieved from
    //  the API profile, defaulting to FORMAT_BINARY.
    //----------------------------------------------------------------
    void setFormat(int format);

    //----------------------------------------------------------------
    // Set the feedback code.
    //  If FEEDBACK_PROFILE is specified then code is retrieved from
    //  the API profile, defaulting to FEEDBACK_NONE.
    //----------------------------------------------------------------
    void setFeedback(int feedback);

    //----------------------------------------------------------------
    // Set the expiry interval.
    //  If EXPIRY_PROFILE is specified then interval is retrieved from
    //  the API profile, defaulting to EXPIRY_NEVER.
    //----------------------------------------------------------------
    void setExpiry(int expiry);

    //----------------------------------------------------------------
    // Set the code page.
    //  If CODEPAGE_PROFILE is specified then codepage number is
    //  retrieved from the API profile, defaulting to CODEPAGE_USER
    //----------------------------------------------------------------
    void setCodepage(int codepage);

    //----------------------------------------------------------------
    // Set the encoding.
    //  If ENCODING_PROFILE is specified then encoding is
    //  retrieved from the API profile, defaulting to ENCODING_DEFAULT
    //----------------------------------------------------------------
    void setEncoding(int encoding);

    //----------------------------------------------------------------
    // Set the format name to the specified string and format name to
    // FORMAT_USENAME.
    //----------------------------------------------------------------
    void setFormatName(const char* name);

    //----------------------------------------------------------------
    // Set the message name.
    // Defines a message name passed on to the Application Management
    // inline service. If left blank, Roma places the name of the last
    // received message on this theme into the message being sent.
    //----------------------------------------------------------------
    void setMsgName(const char* name);

    //----------------------------------------------------------------
    // Set message and tag data.
    //----------------------------------------------------------------
    bool setMsgBuffer(const void* buffer, int dataLength);
    bool setTagBuffer(const void* buffer, int dataLength);

    //----------------------------------------------------------------
    // Sets the put selector.
    //
    // The selector specifies which identifiers contained in the
    // RomaMessage (msg id, correl id) to use when the message
    // is sent.
    //
    // The selector is specified by one of the following constants:
    //   USE_NONE     - doesn't set any specific selector
    //   USE_MSGID    - use Message Id as the selector
    //   USE_CORRELID - use Correlation Id as the selector
    //   USE_MSGID_AND_CORRELID - use both
    //----------------------------------------------------------------
    void setSelector(Roma::PutSelector selector);

    //----------------------------------------------------------------
    // Allows/disallows putting messages under transaction to a
    // non-transactional queue.
    //----------------------------------------------------------------
    void setNoSyncAllowed(bool flag = true);

    //----------------------------------------------------------------
    // Set an API profile name. Supply null name to suppress use.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Sets an error profile name.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // If true specified, Roma does not put the message to exception
    // message service on error.
    //----------------------------------------------------------------
    void setNoEms(bool flag = true);

    //----------------------------------------------------------------
    // If notification is allowed, Roma can send responses straight
    // back to the client, ignoring any further business components.
    // (API profile may override)
    // The following method allows the application to specify that
    // notification messages may NOT be sent back to the client.
    //----------------------------------------------------------------
    void setNoNotification(bool flag = true);

    //----------------------------------------------------------------
    // Indicates that the message should be sent as a notification.
    //----------------------------------------------------------------
    void setNotification(bool flag = true);

    //----------------------------------------------------------------
    // If true, Roma does not send the message to the
    // Transformation Business Service if the next component in the
    // workflow cannot understand the message format being sent.
    // If the recipient will not accept this message format, and this
    // option is selected, Roma treats the Put operation as an error.
    //----------------------------------------------------------------
    void setNoTransformation(bool flag = true);

    //----------------------------------------------------------------
    // The Application Management inline service can return an
    // indication that the application is no longer needed.  (e.g. if
    // the workload has decreased such that some instances of the
    // application are no longer needed.) Only applies to servers.
    //
    // This method allows a Business Element to register to receive
    // these indications (which can still be ignored).
    //----------------------------------------------------------------
    void setTermSupported(bool termAllowed = true);

    //----------------------------------------------------------------
    // This method allows an application to prevent the intervention
    // of any Error Analysis Routines defined in the configuration.
    //----------------------------------------------------------------
    void setSuppressErrorProfile(bool suppressEP = true);

    //----------------------------------------------------------------
    // Indicates whether Roma must increment the message retry count.
    // The counter can be inspected with queryTheme() to determine
    // the number of attempts to send the message.
    //----------------------------------------------------------------
    void setIncrementRetryCount(bool enable = true);

    //----------------------------------------------------------------
    // Set the ILS Data definition to be used with this request.
    //----------------------------------------------------------------
    void setILSData(const RomaILSData& ILSDataSet);

    //----------------------------------------------------------------
    // Get any inline service data related to this response.
    //----------------------------------------------------------------
    RomaILSData* getILSData() const;

    //----------------------------------------------------------------
    // Set a syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Marshal the response.
    //
    // This method is intended for overriding in derived classes.
    // The RomaBusinessElement::put() method calls this method
    // before putting the response. If method returns false or
    // message/tag buffer is in fail state then put() method
    // doesn't put the response and returns an error.
    //----------------------------------------------------------------
    virtual bool marshalResponse();

    //----------------------------------------------------------------
    // Constructor for internal use only.
    //----------------------------------------------------------------
    RomaBeResponse(RomaImplElementResponse*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplElementResponse* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOBRS_HPP

