//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoem.hpp,v 1.15 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoem.hpp,v $
// $Revision: 1.15 $
//--------------------------------------------------------------------
// The RomaExceptionMessage, the RomaExceptionOptions and
// the RomaReportOptions public interfaces.
//--------------------------------------------------------------------

#ifndef __KXCOEM_HPP
#define __KXCOEM_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcocrs.hpp>
#include <kxcocrq.hpp>
#include <kxcobrq.hpp>
#include <kxcobrs.hpp>
#include <kxcosbl.hpp>
#include <kxcopbl.hpp>

//------------------------------------------------------------------------
// The internal implementation classes.
//------------------------------------------------------------------------
class RomaImplExceptionMessage;
class RomaImplExceptionOptions;
class RomaImplReportOptions;


//--------------------------------------------------------------------
// The RomaExceptionMessage class.
//
// Used to report erroneous messages to an Exception
// Message Service using putException() or reportError() methods.
//--------------------------------------------------------------------
class KXCODLL RomaExceptionMessage : public RomaTypeBase, 
                                     public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaExceptionMessage();

    //----------------------------------------------------------------
    // Constructor initializing the exception with user data.
    //----------------------------------------------------------------
    RomaExceptionMessage(const void* buffer, int length);

    //----------------------------------------------------------------
    // Constructors creating the object from an exsisting message.
    //----------------------------------------------------------------
    RomaExceptionMessage(const RomaClientRequest&  message);
    RomaExceptionMessage(const RomaClientResponse& message);
    RomaExceptionMessage(const RomaBeRequest&      message);
    RomaExceptionMessage(const RomaBeResponse&     message);
    RomaExceptionMessage(const RomaStorable&       message);
    RomaExceptionMessage(const RomaPublication&    message);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaExceptionMessage();

    //----------------------------------------------------------------
    // Copy and assignment.
    //
    // Note: these methods don't create an independent copy of the
    // message, but just share the common implementation instance.
    //----------------------------------------------------------------
    RomaExceptionMessage(const RomaExceptionMessage&);
    RomaExceptionMessage& operator = (const RomaExceptionMessage&);

    //----------------------------------------------------------------
    // Specify which elements are present in the exception message to
    // be passed to Roma.
    //
    // Any combination of the following flags can be used:
    //   Values set automatically by Roma if these are requested ...
    //     ELEMENT_TIMESTAMP
    //     ELEMENT_CONFIG
    //     ELEMENT_SYSCONFIG
    //   Values must be supplied by the user if these are set ...
    //     ELEMENT_ERROR
    //     ELEMENT_USERDATA
    //     ELEMENT_MESSAGE
    //
    // Alternatively one of the following combination settings:
    //     ELEMENT_NONE    - no information will be passed
    //     ELEMENT_ALL_PUT - all information will be passed
    //
    // putException() uses the base RomaMessage object, independent
    // of whether ELEMENT_MESSAGE flag is set or not.
    //
    // These settings can be overridden by an API profile.
    //----------------------------------------------------------------
    void setElements(KXCULONG elements);

    //----------------------------------------------------------------
    // Set the call type, which identifies the failing API call, e.g.
    // GETREQUEST, PUTRESPONSE etc.
    //----------------------------------------------------------------
    void setCallType(Roma::CallType callType);

    //----------------------------------------------------------------
    // Set the various error data.
    //----------------------------------------------------------------
    void setStatus(Roma::ErrorStatus status);

    void setCompCode(Roma::ErrorSubclass c,
                     Roma::CompCode compCode);

    void setReturnCode(Roma::ErrorSubclass c,
                       int returnCode);

    void setReason(Roma::ErrorSubclass c,
                   int reason);

    void setOperationType(Roma::ErrorSubclass c,
                          int operationType);

    void setPosition(Roma::ErrorSubclass c,
                     int position);

    void setErrorType(Roma::ErrorSubclass c,
                      Roma::ErrorType errorType);

    void setAdditionalInfo(Roma::ErrorSubclass c,
                           const char *additional);

    //----------------------------------------------------------------
    // Define the user data. The data is not copied.
    //----------------------------------------------------------------
    bool setUserBuffer(const void* buffer, int dataLength);

    //-----------------------------------------------------------------
    // Get the Roma Theme ID.
    //-----------------------------------------------------------------
    const RomaTheme& theme() const;

    //-----------------------------------------------------------------
    // Get/set the Message ID.
    //-----------------------------------------------------------------
    const RomaMsgId& msgId() const;
    void setMsgId(const RomaMsgId& msgId);

    //-----------------------------------------------------------------
    // Get/set the Correlation ID.
    //-----------------------------------------------------------------
    const RomaMsgId& correlId() const;
    void setCorrelId(const RomaMsgId& correlId);

    //-----------------------------------------------------------------
    // Set the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority
    //-----------------------------------------------------------------
    void setPriority(int priority);

    //-----------------------------------------------------------------
    // Set the message persistence to one of ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //-----------------------------------------------------------------
    void setPersistence(Roma::Persistence persistence);

    //-----------------------------------------------------------------
    // Set the format number.
    //  If FORMAT_PROFILE is specified then format is retrieved from
    //  the API profile, defaulting to FORMAT_BINARY.
    //-----------------------------------------------------------------
    void setFormat(int format);

    //-----------------------------------------------------------------
    // Set the feedback code.
    //  If FEEDBACK_PROFILE is specified then code is retrieved from
    //  the API profile, defaulting to FEEDBACK_NONE.
    //-----------------------------------------------------------------
    void setFeedback(int feedback);

    //-----------------------------------------------------------------
    // Set the expiry interval.
    //  If EXPIRY_PROFILE is specified then interval is retrieved from
    //  the API profile, defaulting to EXPIRY_NEVER.
    //-----------------------------------------------------------------
    void setExpiry(int expiry);

    //-----------------------------------------------------------------
    // Set the codepage.
    //  If CODEPAGE_PROFILE is specified then codepage number is
    //  retrieved from the API profile, defaulting to CODEPAGE_USER
    //-----------------------------------------------------------------
    void setCodepage(int codepage);

    //-----------------------------------------------------------------
    // Set the encoding.
    //  If ENCODING_PROFILE is specified then encoding is
    //  retrieved from the API profile, defaulting to ENCODING_DEFAULT
    //-----------------------------------------------------------------
    void setEncoding(int encoding);

    //-----------------------------------------------------------------
    // Set the format name to the specified string and format name to
    // FORMAT_USENAME.
    //-----------------------------------------------------------------
    void setFormatName(const char* name);

    //-----------------------------------------------------------------
    // MESSAGE NAME
    // Defines a message name passed on to the Application Management
    // inline service. If left blank, Roma places the name of the last
    // received message on this theme into the message being sent.
    //-----------------------------------------------------------------
    void setMsgName(const char* name);

    //-----------------------------------------------------------------
    // Set the message data buffer.
    //-----------------------------------------------------------------
    bool setMsgBuffer(const void* buffer, int dataLength);

    //-----------------------------------------------------------------
    // Set the tag data buffer.
    //-----------------------------------------------------------------
    bool setTagBuffer(const void* buffer, int dataLength);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplExceptionMessage* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaExceptionOptions class.
//
// The put options used to send an exception message.
//--------------------------------------------------------------------
class KXCODLL RomaExceptionOptions : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaExceptionOptions();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaExceptionOptions();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaExceptionOptions(const RomaExceptionOptions&);
    RomaExceptionOptions& operator = (const RomaExceptionOptions&);

    //----------------------------------------------------------------
    // Set the business service name.
    // The putException() method will put the message to the specified
    // business service.
    //----------------------------------------------------------------
    void setServiceName(const char* serviceName);

    //----------------------------------------------------------------
    // Set theme options.
    //
    // If a new theme is requested, put() uses a new theme for each
    // request. Otherwise the specified theme will be used.
    //
    // If close theme is requested, the theme is closed after the
    // request is sent. If retain theme is requested, the theme is
    // retained for future use.
    //
    // These selections may be overidden if an API profile is defined
    //----------------------------------------------------------------
    void setNewTheme(bool f = true);
    void setUseTheme(bool f = true);
    void setRetainTheme(bool f = true);
    void setCloseTheme(bool f = true);

    //----------------------------------------------------------------
    // Set the theme to use when an exception message is sent.
    //----------------------------------------------------------------
    void setTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Set the put selector.
    //
    // The selector specifies which identifiers contained in the
    // message (msg id, correl id) to use when the message is sent.
    //
    // The selector is specified by one of the following constants:
    //   USE_NONE     - doesn't set any specific selector
    //   USE_MSGID    - use Message Id as the selector
    //   USE_CORRELID - use Correlation Id as the selector
    //   USE_MSGID_AND_CORRELID - use both
    //----------------------------------------------------------------
    void setSelector(Roma::PutSelector selector);

    //----------------------------------------------------------------
    // Allows/disallows putting messages under transaction to a
    // non-transactional queue.
    //----------------------------------------------------------------
    void setNoSyncAllowed(bool flag = true);

    //----------------------------------------------------------------
    // Set API profile name. Supply null name to suppress use.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Set error profile name. Call setSuppressErrorProfile to
    // suppress use.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // This method allows an application to prevent the intervention
    // of any Error Analysis Routines defined in the configuration.
    //----------------------------------------------------------------
    void setSuppressErrorProfile(bool flag = true);

    //----------------------------------------------------------------
    // If specified, Roma does not put the message to exception
    // message service on error.
    //----------------------------------------------------------------
    void setNoEms(bool flag = true);

    //----------------------------------------------------------------
    // If notification is allowed, Roma can send responses straight
    // back to the client, ignoring any further business components.
    // (API profile may override)
    // The following method allows the application to specify that
    // notification messages may NOT be sent back to the client.
    //----------------------------------------------------------------
    void setNoNotification(bool flag = true);

    //----------------------------------------------------------------
    // If true, Roma does not send the message to the
    // Transformation Business Service if the next component in the
    // workflow cannot understand the message format being sent.
    // If the recipient will not accept this message format, and this
    // option is selected, Roma treats the Put operation as an error.
    //----------------------------------------------------------------
    void setNoTransformation(bool flag = true);

    //----------------------------------------------------------------
    // The Application Management inline service can return an
    // indication that the application is no longer needed.  (e.g. if
    // the workload has decreased such that some instances of the
    // application are no longer needed.) Only applies to servers.
    //
    // This method allows a Business Element to register to receive
    // these indications (which can still be ignored).
    //----------------------------------------------------------------
    void setTermSupported(bool flag = true);

    //----------------------------------------------------------------
    // Indicates whether Roma must increment the message retry count.
    // The counter can be inspected with queryTheme() to determine
    // the number of attempts to send the message.
    //----------------------------------------------------------------
    void setIncrementRetryCount(bool enable = true);

    //----------------------------------------------------------------
    // Set the syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplExceptionOptions* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaReportOptions class.
//
// The put options used to report an error with an exception message.
//--------------------------------------------------------------------
class KXCODLL RomaReportOptions : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaReportOptions();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaReportOptions();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaReportOptions(const RomaReportOptions&);
    RomaReportOptions& operator = (const RomaReportOptions&);

    //----------------------------------------------------------------
    // Assigns error profile name.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // Set default actions.
    //
    // It is the following constants:
    //   ACTION_DEFAULT - error profile or default action will be done
    //
    // Or any combination of the following flags:
    //   ACTION_NOTIFY   - send notification response
    //   ACTION_EMS      - put to exception message service
    //   ACTION_ALERT    - send alert
    //   ACTION_LOG      - write to log
    //   ACTION_RESET    - reset error
    //   ACTION_NOACTION - no action
    //   ACTION_EMSRAW
    //----------------------------------------------------------------
    void setDefaultActions(KXCULONG actions);

    //----------------------------------------------------------------
    // If true, Roma will not allow to send alert.
    //----------------------------------------------------------------
    void setNoAlert(bool flag = true);

    //----------------------------------------------------------------
    // If true, Roma will not allow the option to send the message
    // to the Exception Message Queue if the report error operation
    // fails.
    //----------------------------------------------------------------
    void setNoEms(bool flag = true);

    //----------------------------------------------------------------
    // If true, Roma will not allow the option to send a
    // notification message back to the client if the report
    // error fails.
    //----------------------------------------------------------------
    void setNoNotification(bool flag = true);

    //----------------------------------------------------------------
    // If true and a message is sent out as part of error profiling
    // (either by EMS or as NOTIFY msg) then don't allow this message
    // to be passed to the transformation engine if the msgtype is
    // not recognised.
    //----------------------------------------------------------------
    void setNoTransformation(bool flag = true);

    //----------------------------------------------------------------
    // Set the syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);

    //----------------------------------------------------------------
    // Set theme to use when the ExceptionMessage is reported
    //----------------------------------------------------------------
    void setTheme(const RomaTheme& theme);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplReportOptions* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOEM_HPP

