//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoils.hpp,v 1.14 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoils.hpp,v $
// $Revision: 1.14 $
//--------------------------------------------------------------------
// The RomaILSData public interface.
//--------------------------------------------------------------------

#ifndef __KXCOILS_HPP
#define __KXCOILS_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplILSData;


//--------------------------------------------------------------------
// The RomaILSData class.
// 
// This class allows an application to define data for exchange with
// the specified Inline Services.
//--------------------------------------------------------------------
class KXCODLL RomaILSData : public RomaTraceable,
                            public RomaAllocator
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaILSData();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaILSData();

    //----------------------------------------------------------------
    // Copy constructor and assignment operator.
    //
    // Note: this method does not create an independent copy of the
    // object, but just share the common implementation instance.
    //----------------------------------------------------------------
    RomaILSData(const RomaILSData&);
    RomaILSData& operator=(const RomaILSData&);

    //----------------------------------------------------------------
    // Define a buffer for the ILS identified by the ILSNumber and the
    // ILSType. The method returns true if the parameters are
    // accepted.
    //----------------------------------------------------------------
    bool setBuffer(void* buffer, 
                   int dataLength, 
                   int ILSNumber,
                   Roma::ILSType type = Roma::ILS_TYPE_USER);

    //----------------------------------------------------------------
    // By default an existing buffer is not replaced.
    //----------------------------------------------------------------
    void setReplace(bool replace = true);

    //----------------------------------------------------------------
    // Returns a pointer to the data associated with the ILS defined
    // by ILSNumber and ILSType. The length of the data is returned in
    // the length parameter. The method returns NULL if no data is
    // avaialable.
    //----------------------------------------------------------------
    const void* getBuffer(int& dataLength, 
                          int ILSNumber,
                          Roma::ILSType type = Roma::ILS_TYPE_USER);

    //----------------------------------------------------------------
    // Returns a pointer to the data associated with each ILS defined
    // by ILSType in turn. The length of the data is returned in the
    // length parameter. The method returns NULL if/when no more data
    // is avaialable.
    //----------------------------------------------------------------
    const void* getNextBuffer(int& dataLength, 
                              int& ILSNumber, 
                              Roma::ILSType& type);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Constructor for internal use only.
    //----------------------------------------------------------------
    RomaILSData(RomaImplILSData*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplILSData* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOILS_HPP

